@@ -27,6 +27,7 @@ import errno
 import glob
 import ConfigParser
 from string import Template
+from distutils.dir_util import copy_tree
 
 
 import click
@@ -67,74 +68,104 @@ def version():
 @cli.command()
 @click.option('--config', default='pb_tool.cfg',
               help='Name of the config file to use if other than pb_tool.cfg')
-def deploy(config):
-    """Deploy the plugin using parameters in pb_tool.cfg"""
-    deploy_files(config)
+@click.option('--quick', '-q', is_flag=True,
+              help='Do a quick install without compiling ui, resource, docs, \
+              and translation files')
+def deploy(config, quick):
+    """Deploy the plugin to QGIS plugin directory using parameters in pb_tool.cfg"""
+    deploy_files(config, quick)
 
 
-def deploy_files(config):
+def deploy_files(config, quick):
     """Deploy the plugin using parameters in pb_tool.cfg"""
     # check for the config file
     if not os.path.exists(config):
         click.echo("Configuration file {} is missing.".format(config))
     else:
-        print """Deploying will:
-        * Remove your currently deployed version
-        * Compile the ui and resource files
-        * Build the help docs
-        * Copy everything to your .qgis2/python/plugins directory
-        """
+        cfg = get_config(config)
+        plugin_dir = os.path.join(get_plugin_directory(), cfg.get('plugin', 'name'))
+        if quick:
+            print "Doing quick deployment"
+            install_files(plugin_dir, cfg)
+            print ("Quick deployment complete---if you have problems with your"
+                   " plugin, try doing a full deploy.")
 
-        if click.confirm("Proceed?"):
+        else:
+            print """Deploying will:
+            * Remove your currently deployed version
+            * Compile the ui and resource files
+            * Build the help docs
+            * Copy everything to your .qgis2/python/plugins directory
+            """
 
-            cfg = get_config(config)
-            plugin_dir = os.path.join(get_plugin_directory(), cfg.get('plugin', 'name'))
-            # clean the deployment
-            clean_deployment(False, config)
-            print "Deploying to {}".format(plugin_dir)
-            # compile to make sure everything is fresh
-            print 'Compiling to make sure install is clean'
-            compile_files(cfg)
-            build_docs()
-            install_files = get_install_files(cfg)
-            # make the plugin directory if it doesn't exist
-            if not os.path.exists(plugin_dir):
-                os.mkdir(plugin_dir)
+            if click.confirm("Proceed?"):
 
-            fail = False
+                # clean the deployment
+                clean_deployment(False, config)
+                print "Deploying to {}".format(plugin_dir)
+                # compile to make sure everything is fresh
+                print 'Compiling to make sure install is clean'
+                compile_files(cfg)
+                build_docs()
+                install_files(plugin_dir, cfg)
+
+
+def install_files(plugin_dir, cfg):
+        errors = []
+        install_files = get_install_files(cfg)
+        # make the plugin directory if it doesn't exist
+        if not os.path.exists(plugin_dir):
+            os.mkdir(plugin_dir)
+
+        fail = False
+        for file in install_files:
+            print ("Copying {}".format(file)),
             try:
-                for file in install_files:
-                    print "Copying {}".format(file)
-                    shutil.copy(file, os.path.join(plugin_dir, file))
-                # copy extra dirs
-            except OSError as oops:
-                print "Error copying files: {}, {}".format(file, oops.strerror)
+                shutil.copy(file, os.path.join(plugin_dir, file))
+                print ""
+            except Exception as oops:
+                errors.append("Error copying files: {}, {}".format(
+                    file, oops.strerror))
+                click.echo(click.style(' ----> ERROR', fg='red'))
                 fail = True
+            extra_dirs = cfg.get('files', 'extra_dirs').split()
+            #print "EXTRA DIRS: {}".format(extra_dirs)
+        for xdir in extra_dirs:
+            print "Copying contents of {} to {}".format(xdir, plugin_dir),
             try:
-                extra_dirs = cfg.get('files', 'extra_dirs').split()
-                #print "EXTRA DIRS: {}".format(extra_dirs)
-                for xdir in extra_dirs:
-                    print "Copying contents of {} to {}".format(xdir, plugin_dir)
-                    shutil.copytree(xdir, "{}/{}".format(
-                        plugin_dir, xdir))
-            except OSError as oops:
-                print "Error copying directory: {}, {}".format(xdir, oops.strerror)
+                copy_tree(xdir, "{}/{}".format(plugin_dir, xdir))
+                print ""
+            except Exception as oops:
+                errors.append("Error copying directory: {}, {}".format(
+                    xdir, oops.message))
+                click.echo(click.style(' ----> ERROR', fg='red'))
                 fail = True
-            try:
-                help_src = cfg.get('help', 'dir')
-                help_target = os.path.join(
-                    get_plugin_directory(),
-                    cfg.get('plugin', 'name'),
-                    cfg.get('help', 'target'))
-                print "Copying {} to {}".format(help_src, help_target)
-                shutil.copytree(help_src, help_target)
-            except OSError as oops:
-                print "Error copying help files: {}, {}".format(help_src, oops.strerror)
-                fail = True
-            if fail:
-                print "\nOne or more files/directories failed to deploy."
-                print "To ensure proper deployment, try using dclean to delete"
-                print "the plugin before deploying."
+        help_src = cfg.get('help', 'dir')
+        help_target = os.path.join(
+            get_plugin_directory(),
+            cfg.get('plugin', 'name'),
+            cfg.get('help', 'target'))
+        print "Copying {} to {}".format(help_src, help_target),
+        #shutil.copytree(help_src, help_target)
+        try:
+            copy_tree(help_src, help_target)
+            print ""
+        except Exception as oops:
+            errors.append("Error copying help files: {}, {}".format(
+                help_src, oops.message))
+            click.echo(click.style(' ----> ERROR', fg='red'))
+            fail = True
+        if fail:
+            print "\nERRORS:"
+            for error in errors:
+                print error
+            print ""
+            print "One or more files/directories specified in your config"
+            print "file failed to deploy---make sure they exist or if not"
+            print "needed remove them from the config."
+            print "To ensure proper deployment, make sure your UI and resource"
+            print "files are compiled. Using dclean to delete"
+            print "the plugin before deploying may also help."
 
 
 def clean_deployment(ask_first=True, config='pb_tool.cfg'):
@@ -236,6 +267,41 @@ def build_docs():
         print "No help directory exists in the current directory"
 
 
+@cli.command()
+@click.option('--config', default='pb_tool.cfg',
+              help='Name of the config file to use if other than pb_tool.cfg')
+def translate(config):
+    """ Build translations using lrelease. Locales must be specified
+    in the config file and the corresponding .ts file must exist in
+    the i18n directory of your plugin."""
+    possibles = ['lrelease', 'lrelease-qt4']
+    for binary in possibles:
+        cmd = check_path(binary)
+        if cmd:
+            break
+    if not cmd:
+        print ("Unable to find the lrelease command. Make sure it is installed"
+               "  and in your path.")
+        if sys.platform == 'win32':
+            print ('You can get lrelease by installing'
+                   ' the qt4-devel package in the Libs'
+                   '\nsection of the OSGeo4W Advanced Install.')
+    else:
+        cfg = get_config(config)
+        if check_cfg(cfg, 'files', 'locales'):
+            locales = cfg.get('files', 'locales').split()
+            if locales:
+                for locale in locales:
+                    (name, ext) = os.path.splitext(locale)
+                    if ext != '.ts':
+                        print 'no ts extension'
+                        locale = name + '.ts'
+                    print cmd, locale
+                    subprocess.check_call([cmd, os.path.join('i18n', locale)])
+            else:
+                print "No translations are specified in {}".format(config)
+
+
 @cli.command()
 @click.option('--config', default='pb_tool.cfg',
               help='Name of the config file to use if other than pb_tool.cfg')
@@ -335,7 +401,7 @@ def create(name):
 
     # get the list of python files
     py_files = glob.glob('*.py')
-    
+
     # guess the main dialog ui file
     main_dlg = glob.glob('*_dialog_base.ui')
 
@@ -354,12 +420,18 @@ def create(name):
 
     extras = glob.glob('*.png') + glob.glob('metadata.txt')
 
+    locale_list = glob.glob('i18n/*.ts')
+    locales = []
+    for locale in locale_list:
+        locales.append(os.path.basename(locale))
+
     cfg = template.substitute(Name=cfg_name,
                               PythonFiles=' '.join(py_files),
                               MainDialog=' '.join(main_dlg),
                               CompiledUiFiles=' '.join(other_ui),
                               Resources=' '.join(resources),
-                              Extras=' '.join(extras))
+                              Extras=' '.join(extras),
+                              Locales=' '.join(locales))
 
     fname = name
     if os.path.exists(fname):
@@ -435,31 +507,49 @@ def compile_files(cfg):
     # TODO add changed detection
     #cfg = get_config(config)
 
-    ui_files = cfg.get('files', 'compiled_ui_files').split()
-    ui_count = 0
-    for ui in ui_files:
-        if os.path.exists(ui):
-            (base, ext) = os.path.splitext(ui)
-            output = "{}.py".format(base)
-            print "Compiling {} to {}".format(ui, output)
-            subprocess.check_call(['pyuic4', '-o', output, ui])
-            ui_count += 1
-        else:
-            print "{} does not exist---skipped".format(ui)
-    print "Compiled {} UI files".format(ui_count)
+    # check to see if we have pyuic4
+    pyuic4 = check_path('pyuic4')
 
-    res_files = cfg.get('files', 'resource_files').split()
-    res_count = 0
-    for res in res_files:
-        if os.path.exists(res):
-            (base, ext) = os.path.splitext(res)
-            output = "{}_rc.py".format(base)
-            print "Compiling {} to {}".format(res, output)
-            subprocess.check_call(['pyrcc4', '-o', output, res])
-            res_count += 1
-        else:
-            print "{} does not exist---skipped".format(res)
-    print "Compiled {} resource files".format(res_count)
+    if not pyuic4:
+        print "pyuic4 is not in your path---unable to compile your ui files"
+    else:
+        ui_files = cfg.get('files', 'compiled_ui_files').split()
+        ui_count = 0
+        for ui in ui_files:
+            if os.path.exists(ui):
+                (base, ext) = os.path.splitext(ui)
+                output = "{}.py".format(base)
+                if file_changed(ui, output):
+                    print "Compiling {} to {}".format(ui, output)
+                    subprocess.check_call([pyuic4, '-o', output, ui])
+                    ui_count += 1
+                else:
+                    print "Skipping {} (unchanged)". format(ui)
+            else:
+                print "{} does not exist---skipped".format(ui)
+        print "Compiled {} UI files".format(ui_count)
+
+    # check to see if we have pyrcc4
+    pyrcc4 = check_path('pyrcc4')
+
+    if not pyrcc4:
+        print "pyrcc4 is not in your path---unable to compile your resource file(s)"
+    else:
+        res_files = cfg.get('files', 'resource_files').split()
+        res_count = 0
+        for res in res_files:
+            if os.path.exists(res):
+                (base, ext) = os.path.splitext(res)
+                output = "{}_rc.py".format(base)
+                if file_changed(res, output):
+                    print "Compiling {} to {}".format(res, output)
+                    subprocess.check_call([pyrcc4, '-o', output, res])
+                    res_count += 1
+                else:
+                    print "Skipping {} (unchanged)". format(res)
+            else:
+                print "{} does not exist---skipped".format(res)
+        print "Compiled {} resource files".format(res_count)
 
 
 def copy(source, destination):
@@ -476,7 +566,8 @@ def copy(source, destination):
 
     """
     try:
-        shutil.copytree(source, destination)
+        #shutil.copytree(source, destination)
+        copy_tree(source, destination)
     except OSError as e:
         # If the error was caused because the source wasn't a directory
         if e.errno == errno.ENOTDIR:
@@ -531,6 +622,9 @@ extras: $Extras
 # These must be subdirectories under the plugin directory
 extra_dirs:
 
+# ISO code(s) for any locales (translations), separated by spaces.
+# Corresponding .ts files must exist in the i18n directory
+locales: $Locales
 
 [help]
 # the built help directory that should be deployed with the plugin
@@ -539,3 +633,40 @@ dir: help/build/html
 target: help
 """
     return template
+
+
+def check_path(app):
+    """ Adapted from StackExchange:
+        http://stackoverflow.com/questions/377017
+    """
+    import os
+
+    def is_exe(fpath):
+        return os.path.exists(fpath) and os.access(fpath, os.X_OK)
+
+    def ext_candidates(fpath):
+        yield fpath
+        for ext in os.environ.get("PATHEXT", "").split(os.pathsep):
+            yield fpath + ext
+
+    fpath, fname = os.path.split(app)
+    if fpath:
+        if is_exe(app):
+            return app
+    else:
+        for path in os.environ["PATH"].split(os.pathsep):
+            exe_file = os.path.join(path, app)
+            for candidate in ext_candidates(exe_file):
+                if is_exe(candidate):
+                    return candidate
+
+    return None
+
+
+def file_changed(infile, outfile):
+    try:
+        infile_s = os.stat(infile)
+        outfile_s = os.stat(outfile)
+        return infile_s.st_mtime > outfile_s.st_mtime
+    except:
+        return True
